// Global vars
var gPref = Components.classes["@mozilla.org/preferences-service;1"]
               .getService(Components.interfaces.nsIPrefService)
               .QueryInterface(Components.interfaces.nsIPrefBranch2);
var updateTimer = null;
var numGroups = 0;

function Startup() {
  refreshGroupsList();
  SpeedDialPrefObserver.addPrefObserver();
}

function Unload() {
  SpeedDialPrefObserver.removePrefObserver();
}

function refreshGroupsList() {
  var list = document.getElementById("groupsList");
  var oldSelectedIndex = list.selectedIndex;
  
  while (list.firstChild) {
    list.removeChild(list.firstChild);
  }

  numGroups = gPref.getIntPref("extensions.speeddial.numGroups");
  var speedDialBundle = document.getElementById("bundle_speeddial");

  for (var b=1; b<=numGroups; b++) {
    var row = document.createElement("listitem");
    row.id = "";
    row.setAttribute("groupnumber", b);
    if (gPref.prefHasUserValue("extensions.speeddial.group-" + b + "-title")) {
      row.setAttribute("label", gPref.getComplexValue("extensions.speeddial.group-" + b + "-title", Components.interfaces.nsISupportsString).data);
    } else {
      row.setAttribute("label", speedDialBundle.getFormattedString("untitledGroup.label", [b]));
    }
    list.appendChild(row);
  }
  
  if (oldSelectedIndex < numGroups) {
    list.selectedIndex = oldSelectedIndex;
  }
}

function groupsListSelect(e) {
  var moveUpButton = document.getElementById("moveUpButton");
  var moveDownButton = document.getElementById("moveDownButton");
  var editButton = document.getElementById("editButton");
  var removeButton = document.getElementById("removeButton");
  var list = document.getElementById("groupsList");
  if (list.selectedItem) {
    if (list.selectedIndex == 0) {
      moveUpButton.setAttribute("disabled", "true");
    } else {
      moveUpButton.setAttribute("disabled", "false");
    }
    if (list.selectedIndex == (numGroups - 1)) {
      moveDownButton.setAttribute("disabled", "true");
    } else {
      moveDownButton.setAttribute("disabled", "false");
    }
    editButton.setAttribute("disabled", "false");
    removeButton.setAttribute("disabled", "false");
  } else {
    moveUpButton.setAttribute("disabled", "true");
    moveDownButton.setAttribute("disabled", "true");
    editButton.setAttribute("disabled", "true");
    removeButton.setAttribute("disabled", "true");
  }
}

function moveUpGroup() {
  var list = document.getElementById("groupsList");
  SpeedDialUtils.swapGroup(list.selectedIndex, (list.selectedIndex + 1));
  list.selectedIndex--;
}

function moveDownGroup() {
  var list = document.getElementById("groupsList");
  SpeedDialUtils.swapGroup((list.selectedIndex + 1), (list.selectedIndex + 2));
  list.selectedIndex++;
}

function editCurrentGroup() {
  var list = document.getElementById("groupsList");
  openDialog("chrome://speeddial/content/editGroup.xul", "",
             "centerscreen,chrome,dialog,resizable,dependent", [ (list.selectedIndex + 1) ]);
}

function removeCurrentGroup() {
  var list = document.getElementById("groupsList");
  SpeedDialUtils.removeGroup((list.selectedIndex + 1));
}

function addNewGroup() {
  openDialog("chrome://speeddial/content/editGroup.xul", "",
             "centerscreen,chrome,dialog,resizable,dependent", [ -1 ]);
}

var SpeedDialPrefObserver = 
{
  prefObserver : {
    observe: function(subject, topic, data) {
    // subject is the nsIPrefBranch we're observing (after appropriate QI)
    // data is the name of the pref that's been changed (relative to subject)
    var updateAllThumbnails = false;
    var updatePendingThumbnails = false;

    if (topic == "nsPref:changed") {
      if (data.indexOf("extensions.speeddial.group-") == 0) {
        if (updateTimer == null) {
          updateTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
        }
        updateTimer.cancel();
        updateTimer.initWithCallback(SpeedDialPrefObserver.prefObserver, 100, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
      }
    }
  },

  QueryInterface : function (aIID) {
    if (aIID.equals(Components.interfaces.nsIObserver) || 
    aIID.equals(Components.interfaces.nsITimerCallback) ||
    aIID.equals(Components.interfaces.nsISupports) ||
    aIID.equals(Components.interfaces.nsISupportsWeakReference))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

    notify: function(timer) {
      refreshGroupsList();
    }
  },
  
  addPrefObserver : function () {
    if (!gPref) return;
    gPref.addObserver("extensions.speeddial.", SpeedDialPrefObserver.prefObserver, true);
  },

  removePrefObserver : function () {
    if (!gPref) return;
    gPref.removeObserver("extensions.speeddial.", SpeedDialPrefObserver.prefObserver);
  }
};
